//
// DO WHAT YOU WANT TO PUBLIC LICENSE
//
// Copyright (C) 2012 James Custer
//
// Everyone is permitted to copy and distribute verbatim or modified
// copies of this license document.
//
// This program is free software. It comes without any warranty, to
// the extent permitted by applicable law.
//
// TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
//
// 0. You just DO WHAT YOU WANT TO.
//

//-----------------------------------------------------------------------------
// This is an example of how NOT to use boost::mpi's blocking
// send/recv routines. This program may deadlock, depending on your
// MPI configuration.
//
// Notice that the first thing both processes do is perform a (blocking) send.
// Since they both block on the send, the program might deadlock.
// We say might since the actual behavior depends on the implementation of MPI.
//
// To fix this problem, we one process should send and then receive,
// while the other does the opposite.
//-----------------------------------------------------------------------------

#include <iostream>
#include <string>

#include <boost/mpi.hpp>

int main(int argc, char* argv[])
{
  boost::mpi::environment env(argc, argv);
  boost::mpi::communicator world;

  if (world.size() != 2)
  {
    std::cerr << "You must execute the program with 2 processes." << std::endl;
    return -1;
  }

  std::string message;

  if (world.rank() == 0)
  {
    world.send(1, 0, std::string("Hello 1, from 0"));
    world.recv(1, 1, message);
  }
  else
  {
    world.send(0, 1, std::string("Hello 0, from 1"));
    world.recv(0, 0, message);
  }

  std::cout << message << std::endl;
}
