//
// DO WHAT YOU WANT TO PUBLIC LICENSE
//
// Copyright (C) 2012 James Custer
//
// Everyone is permitted to copy and distribute verbatim or modified
// copies of this license document.
//
// This program is free software. It comes without any warranty, to
// the extent permitted by applicable law.
//
// TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
//
// 0. You just DO WHAT YOU WANT TO.
//

//-----------------------------------------------------------------------------
// This is an example of how to use boost::mpi's blocking
// send/recv routines without causing a deadlock.
//
// Note that one process performs a send and then a recv
// and the other process performs a recv and then a send.
//-----------------------------------------------------------------------------

#include <iostream>
#include <string>

#include <boost/mpi.hpp>

int main(int argc, char* argv[])
{
  boost::mpi::environment env(argc, argv);
  boost::mpi::communicator world;

  if (world.size() != 2)
  {
    std::cerr << "You must execute the program with 2 processes." << std::endl;
    return -1;
  }

  std::string message;

  if (world.rank() == 0)
  {
    world.send(1, 0, std::string("Hello 1, from 0"));
    world.recv(1, 1, message);
  }
  else
  {
    world.recv(0, 0, message);
    world.send(0, 1, std::string("Hello 0, from 1"));
  }

  std::cout << message << std::endl;
}
