//
// DO WHAT YOU WANT TO PUBLIC LICENSE
//
// Copyright (C) 2012 James Custer
//
// Everyone is permitted to copy and distribute verbatim or modified
// copies of this license document.
//
// This program is free software. It comes without any warranty, to
// the extent permitted by applicable law.
//
// TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
//
// 0. You just DO WHAT YOU WANT TO.
//

//-----------------------------------------------------------------------------
// This is an example of how to use boost::mpi's non-blocking
// isend/irecv routines.
//
// Both processes can perform a (non-blocking) isend first and irecv later.
// Compare this to the (blocking) send, where a deadlock can occur depending
// on how we order the send/recv's.
//-----------------------------------------------------------------------------

#include <iostream>
#include <string>
#include <vector>

#include <boost/mpi.hpp>

int main(int argc, char* argv[])
{
  boost::mpi::environment env(argc, argv);
  boost::mpi::communicator world;

  if (world.size() != 2)
  {
    std::cerr << "You must execute the program with 2 processes." << std::endl;
    return -1;
  }

  std::vector<boost::mpi::request> requests;
  std::string message;
  if (world.rank() == 0)
  {
    requests.push_back( world.isend(1, 0, std::string("Hello 1, from 0")) );
    requests.push_back( world.irecv(1, 1, message) );
  }
  else
  {
    requests.push_back( world.isend(0, 1, std::string("Hello 0, from 1")) );
    requests.push_back( world.irecv(0, 0, message) );
  }

  boost::mpi::wait_all(requests.begin(), requests.end());
  // We print the message after we have waited for all
  // the requests to complete.
  std::cout << message << std::endl;
}
